package com.example.camerademo.activity;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ImageView;

import java.io.File;
import java.util.ArrayList;

import com.example.camerademo.R;
import com.example.camerademo.view.CustomCameraView;
import com.example.camerademo.view.CustomCameraView.OnTakePictureInfo;
import com.lidroid.xutils.ViewUtils;
import com.lidroid.xutils.view.annotation.ViewInject;
import com.lidroid.xutils.view.annotation.event.OnClick;

public class CameraActivity extends BaseActivity {

    @ViewInject(R.id.cc_camera)
    private CustomCameraView cameraView;
    @ViewInject(R.id.imv_pic)
    private ImageView imv_pic;

    private Bitmap bitmap = null;
    private String mFilePath = null;
//    private String mPhotoPath;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (!isHasCamera() || !isCameraCanUse()) { //判断是否有摄像头并且可用
            toastMessage(R.string.msg_no_camera);
            onBackPressed();
            return;
        }
        
        setContentView(R.layout.activity_camera);
        ViewUtils.inject(this);
        
        cameraView.setOnTakePictureInfo(new OnTakePictureInfo() {

            @Override
            public void onTakePictureInofo(boolean isSuccess, File file) {
                if (isSuccess) {
                	
                    if (bitmap != null) {
                        if (!bitmap.isRecycled()) {
                            bitmap.recycle();
                            System.gc(); //通知系统进行垃圾回收，但不一定会执行垃圾回收
                        }
                    }

                    mFilePath = file.getAbsolutePath();
                    bitmap = getLoacalBitmap(mFilePath);

                    Log.i("MainActivity", "AbsolutePath=" + file.getAbsolutePath());

                    imv_pic.setImageBitmap(bitmap);
                    imv_pic.setVisibility(View.VISIBLE);

                    // 更新相册
                    updatePhotos(file);

                } else {
                	toastMessage(R.string.msg_take_fail);
                }
            }
        });
    }
    
    //使用xUtils开源框架的注解方式实现事件绑定
    @OnClick({R.id.btn_showcamera, R.id.imv_gallery, R.id.imv_pic})
    private void onMyClick(View v) {
    	switch (v.getId()) {
		case R.id.btn_showcamera: //拍照
			takePhoto();
			break;
		case R.id.imv_gallery: //相片列表
			openPhotoList();
			break;
		case R.id.imv_pic: //相片视图
			 openPhotoView();
			break;

		default:
			break;
		}
    }
    
    /**
     * 调用拍照功能
     */
    private void takePhoto() {
    	try {
            cameraView.takePicture();
        } catch (Exception e) {
            e.printStackTrace();
            toastMessage(R.string.msg_take_fail);
        }
    }
    
    /**
     * 打开相片列表
     */
    private void openPhotoList() {
    	Intent it_gallery = new Intent();
		it_gallery.addCategory(Intent.CATEGORY_OPENABLE);
		it_gallery.setType("image/*");
        startActivity(it_gallery);
    }
    /**
     * 打开相片视图
     */
    private void openPhotoView() {
    	if (mFilePath != null) {
            Intent intent = new Intent(Intent.ACTION_VIEW);
            File file = new File(mFilePath);
            Uri mUri = Uri.fromFile(file);
            intent.setDataAndType(mUri, "image/*");
            startActivity(intent);
        }
    }
    
    /**
     * 发送广播，图库更新照片
     * @param file 新增的图片
     */
    private void updatePhotos(File file) {
    	Intent intent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
        Uri uri = Uri.fromFile(file);
        intent.setData(uri);
        sendBroadcast(intent);
    }

    /**
     *  加载本地图片
     * @param url 文件的uri路径
     * @return 位图对象
     */
    public Bitmap getLoacalBitmap(String url) {
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inJustDecodeBounds = true;
        // 获取这个图片的宽和高
        Bitmap bitmap = BitmapFactory.decodeFile(url, options); // 此时的bitmap为null
        options.inJustDecodeBounds = false;
        // 计算缩放比
        int be = (int) (options.outHeight / (float) 100);
        if (be <= 0)
            be = 1;
        options.inSampleSize = be;
        // 重新读入图片，注意这次要把options.inJustDecodeBounds 设为 false
        bitmap = BitmapFactory.decodeFile(url, options); // 此时的bitmap不为null
        
        return bitmap;  // 把流转化为Bitmap图片
    }
    
    /**
     * 返回指定目录下的所有图片
     * @param path 指定目录
     * @return
     */
    private ArrayList<File> getDefaultPhoto(String path) {
    	File file = new File(path);
    	ArrayList<File> fileList = new ArrayList<File>();
    	
    	if(file.isDirectory()) { //是否文件夹
    		File[] fileArr = file.listFiles();
    		for(int i=0; fileArr != null && i < fileArr.length; i++) {
    			File f = fileArr[i];
    			if(f.isFile()) { //是否是文件
    				int index = f.getPath().lastIndexOf("."); //定位最后一个字符'.'的位置
    				if(index <= 0) continue; //不存在，继续
    				String str = f.getPath().substring(index); //获取文件后缀名
    				if(".jpg".equals(str.toLowerCase())
    						|| ".png".equals(str.toLowerCase())
    						|| ".jpeg".equals(str.toLowerCase())) {
    					fileList.add(f);
    				}
    			}
    		}
    	}
    	return fileList;
    }
    
    private long timeMillis;
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
    	// TODO Auto-generated method stub
    	if(keyCode == KeyEvent.KEYCODE_BACK) {
    		if(System.currentTimeMillis() - timeMillis < 1000) {
    			finish();
    		} else {
    			timeMillis = System.currentTimeMillis();
    			toastMessage(R.string.msg_exit_takephoto);
    			return true;
    		}
    	}
    	return super.onKeyDown(keyCode, event);
    }
}
