package com.example.camerademo.view;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.res.Configuration;
import android.graphics.Rect;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.Size;
import android.os.Environment;
import android.os.Handler;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.example.camerademo.R;
import com.example.camerademo.activity.BaseActivity;

@SuppressLint("ShowToast")
public class CustomCameraView extends FrameLayout implements
SurfaceHolder.Callback, AutoFocusCallback {
	public static final String PHOTO_FILE_NAME = "CameraDemo";
	private Context context = null;
	private Camera camera = null;
	private SurfaceHolder surface_holder = null;
	private SurfaceView surface_camera = null;
	private int viewWidth = 0;
	private int viewHeight = 0;
	private OnTakePictureInfo onTakePictureInfo = null;
	private View view_focus = null;
	private PreviewFrameLayout frameLayout = null;

	/**
	 * 模式 NONE：无 FOCUSING：正在聚焦. FOCUSED:聚焦成功 FOCUSFAIL：聚焦失败
	 */
	private enum MODE {
		NONE, FOCUSING, FOCUSED, FOCUSFAIL

	}

	private MODE mode = MODE.NONE;// 默认模式

	public CustomCameraView(Context context) {
		super(context);
	}

	public CustomCameraView(Context context, AttributeSet attrs) {
		super(context, attrs);
		this.context = context;
		LayoutInflater.from(context).inflate(R.layout.preview_frame, this);
		frameLayout = (PreviewFrameLayout) findViewById(R.id.frame_layout);
		surface_camera = (SurfaceView) findViewById(R.id.camera_preview);
		view_focus = findViewById(R.id.view_focus);
		surface_holder = surface_camera.getHolder();
		surface_holder.addCallback(this);
		frameLayout.setOnTouchListener(onTouchListener);

		surface_holder.setKeepScreenOn(true);
	}

	/**
	 * 设置相机焦距 *
	 */
	private void setZoom(int mValue) {
		Camera.Parameters mParams = camera.getParameters();
		mParams.setZoom(mValue);
		camera.setParameters(mParams);
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		if (((BaseActivity) context).isHasCamera()) {

			camera = getCameraInstance();

			if (camera == null) {
				Toast.makeText(context, R.string.msg_no_camera, Toast.LENGTH_SHORT).show();
				//                ((BaseNoSystemBarTintActivity) context).onBackPressed();
				return;
			}

			try {
				setZoom(camera.getParameters().getMaxZoom() / 2);
				camera.setPreviewDisplay(surface_holder);

			} catch (IOException e) {
			}
			updateCameraParameters();
			camera.startPreview();

		}

	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {

		// try {
		// camera.setPreviewDisplay(surface_holder);
		// updateCameraParameters();
		// camera.startPreview();
		// } catch (IOException e) {
		// // TODO 鑷姩鐢熸垚鐨�catch 鍧�
		// e.printStackTrace();
		// }
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		if (camera != null && holder != null) {
			camera.stopPreview();
			camera.release();
		}

	}

	@Override
	protected void onMeasure(int widthSpec, int heightSpec) {

		viewWidth = MeasureSpec.getSize(widthSpec);
		viewHeight = MeasureSpec.getSize(heightSpec);

		super.onMeasure(
				MeasureSpec.makeMeasureSpec(viewWidth, MeasureSpec.EXACTLY),
				MeasureSpec.makeMeasureSpec(viewHeight, MeasureSpec.EXACTLY));
	}

	private Camera getCameraInstance() {
		Camera c = null;
		try {
			int cameraCount = 0;

			Camera.CameraInfo cameraInfo = new Camera.CameraInfo();
			cameraCount = Camera.getNumberOfCameras(); // get cameras number

			for (int camIdx = 0; camIdx < cameraCount; camIdx++) {
				Camera.getCameraInfo(camIdx, cameraInfo); // get camerainfo
				if (cameraInfo.facing == Camera.CameraInfo.CAMERA_FACING_BACK) { // 浠ｈ〃鎽勫儚澶寸殑鏂逛綅锛岀洰鍓嶆湁瀹氫箟鍊间袱涓垎鍒负CAMERA_FACING_FRONT鍓嶇疆鍜孋AMERA_FACING_BACK鍚庣疆
					try {
						c = Camera.open(camIdx);
					} catch (RuntimeException e) {
					}
				}
			}
			if (c == null) {
				c = Camera.open(0); // attempt to get a Camera instance

			}
		} catch (Exception e) {
			Toast.makeText(context, "摄像头打开失败！", Toast.LENGTH_SHORT);
		}
		if (c == null) {
			Toast.makeText(context, "摄像头打开失败！", Toast.LENGTH_SHORT);
		}
		return c;
	}

	private void updateCameraParameters() {
		if (camera != null) {
			Camera.Parameters p = camera.getParameters();

			long time = new Date().getTime();
			p.setGpsTimestamp(time);

			Size previewSize = findBestPreviewSize(p);
			p.setPreviewSize(previewSize.width, previewSize.height);
			// p.setPictureSize(previewSize.width, previewSize.height);

			// Set the preview frame aspect ratio according to the picture size.

			frameLayout.setAspectRatio((double) previewSize.width
					/ previewSize.height);

			if (context.getResources().getConfiguration().orientation != Configuration.ORIENTATION_LANDSCAPE) {
				camera.setDisplayOrientation(90);
				p.setRotation(90);
			}
			camera.setParameters(p);
		}
	}

	/**
	 * 找到最合适的显示分辨率 （防止预览图像变形）
	 *
	 * @param parameters
	 * @return
	 */
	private Size findBestPreviewSize(Camera.Parameters parameters) {

		//系统支持的所有预览分辨率
		String previewSizeValueString = null;
		previewSizeValueString = parameters.get("preview-size-values");

		Log.d("CameraActivity", "previewSizeValueString="
				+ previewSizeValueString);

		if (previewSizeValueString == null) {
			previewSizeValueString = parameters.get("preview-size-value");
		}

		if (previewSizeValueString == null) { // 鏈変簺鎵嬫満渚嬪m9鑾峰彇涓嶅埌鏀寔鐨勯瑙堝ぇ灏�灏辩洿鎺ヨ繑鍥炲睆骞曞ぇ灏�
			return camera.new Size(getScreenWH().widthPixels,
					getScreenWH().heightPixels);
		}
		float bestX = 0;
		float bestY = 0;

		float tmpRadio = 0;
		float viewRadio = 0;

		if (viewWidth != 0 && viewHeight != 0) {
			viewRadio = Math.min((float) viewWidth, (float) viewHeight)
					/ Math.max((float) viewWidth, (float) viewHeight);
		}

		Log.d("CameraActivity", "viewRadio=" + viewRadio);
		// System.out.println("CustomCameraView previewSizeValueString COMMA_PATTERN = "
		// + previewSizeValueString);

		String[] COMMA_PATTERN = previewSizeValueString.split(",");
		for (String prewsizeString : COMMA_PATTERN) {
			prewsizeString = prewsizeString.trim();

			int dimPosition = prewsizeString.indexOf('x');
			if (dimPosition == -1) {
				continue;
			}

			float newX = 0;
			float newY = 0;

			try {
				newX = Float.parseFloat(prewsizeString
						.substring(0, dimPosition));
				newY = Float.parseFloat(prewsizeString
						.substring(dimPosition + 1));
			} catch (NumberFormatException e) {
				continue;
			}

			float radio = Math.min(newX, newY) / Math.max(newX, newY);

			Log.d("CameraActivity", "radio=" + radio);
			if (tmpRadio == 0) {
				tmpRadio = radio;
				bestX = newX;
				bestY = newY;
			} else if (tmpRadio != 0
					&& (Math.abs(radio - viewRadio)) < (Math.abs(tmpRadio
							- viewRadio))) {
				tmpRadio = radio;
				bestX = newX;
				bestY = newY;
			} else if (tmpRadio != 0
					&& (Math.abs(radio - viewRadio)) == (Math.abs(tmpRadio
							- viewRadio))) {

				if ((Math.abs(newY - viewWidth)) < (Math.abs(bestY - viewWidth))) {
					tmpRadio = radio;
					bestX = newX;
					bestY = newY;
				}
			}
		}

		if (bestX > 0 && bestY > 0) {
			System.out
			.println("CustomCameraView previewSizeValueString bestX = "
					+ bestX + ", bestY = " + bestY);
			return camera.new Size((int) bestX, (int) bestY);
		}
		return null;
	}

	/**
	 * 鐐瑰嚮鏄剧ず鐒︾偣鍖哄煙
	 */
	OnTouchListener onTouchListener = new OnTouchListener() {

		@Override
		public boolean onTouch(View v, MotionEvent event) {
			if (event.getAction() == MotionEvent.ACTION_DOWN) {
				// System.out.println("CustomCameraView view_focus.getWidth() "
				// +
				// view_focus.getWidth() + ",  view_focus.getHeight() = " +
				// view_focus.getHeight());
				// System.out.println("CustomCameraView event.getX() " +
				// event.getX() +
				// ",  event.getY() = " + event.getY());

				int width = view_focus.getWidth();
				int height = view_focus.getHeight();
				view_focus.setBackground(getResources().getDrawable(
						R.drawable.ic_focus_focusing));
				view_focus.setX(event.getX() - (width / 2));
				view_focus.setY(event.getY() - (height / 2));
			} else if (event.getAction() == MotionEvent.ACTION_UP) {
				mode = MODE.FOCUSING;
				focusOnTouch(event);
			}

			return true;
		}
	};

	/**
	 * 璁剧疆鐒︾偣鍜屾祴鍏夊尯鍩�
	 *
	 * @param event
	 */
	public void focusOnTouch(MotionEvent event) {

		int[] location = new int[2];
		frameLayout.getLocationOnScreen(location);

		Rect focusRect = calculateTapArea(view_focus.getWidth(),
				view_focus.getHeight(), 1f, event.getRawX(), event.getRawY(),
				location[0], location[0] + frameLayout.getWidth(), location[1],
				location[1] + frameLayout.getHeight());
		Rect meteringRect = calculateTapArea(view_focus.getWidth(),
				view_focus.getHeight(), 1.5f, event.getRawX(), event.getRawY(),
				location[0], location[0] + frameLayout.getWidth(), location[1],
				location[1] + frameLayout.getHeight());

		Camera.Parameters parameters = camera.getParameters();
		parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);

		// System.out.println("CustomCameraView getMaxNumFocusAreas = " +
				// parameters.getMaxNumFocusAreas());
		if (parameters.getMaxNumFocusAreas() > 0) {
			List<Camera.Area> focusAreas = new ArrayList<Camera.Area>();
			focusAreas.add(new Camera.Area(focusRect, 1000));

			parameters.setFocusAreas(focusAreas);
		}

		// System.out.println("CustomCameraView getMaxNumMeteringAreas = " +
		// parameters.getMaxNumMeteringAreas());
		if (parameters.getMaxNumMeteringAreas() > 0) {
			List<Camera.Area> meteringAreas = new ArrayList<Camera.Area>();
			meteringAreas.add(new Camera.Area(meteringRect, 1000));

			parameters.setMeteringAreas(meteringAreas);
		}

		try {
			camera.setParameters(parameters);

			camera.autoFocus(this);
		} catch (Exception e) {
		}

	}

	/**
	 * 璁＄畻鐒︾偣鍙婃祴鍏夊尯鍩�
	 *
	 * @param focusWidth
	 * @param focusHeight
	 * @param areaMultiple
	 * @param x
	 * @param y
	 * @param previewleft
	 * @param previewRight
	 * @param previewTop
	 * @param previewBottom
	 * @return Rect(left, top, right, bottom) : left銆乼op銆乺ight銆乥ottom鏄互鏄剧ず鍖哄煙涓績涓哄師鐐圭殑鍧愭爣
	 */
	public Rect calculateTapArea(int focusWidth, int focusHeight,
			float areaMultiple, float x, float y, int previewleft,
			int previewRight, int previewTop, int previewBottom) {
		int areaWidth = (int) (focusWidth * areaMultiple);
		int areaHeight = (int) (focusHeight * areaMultiple);
		int centerX = (previewleft + previewRight) / 2;
		int centerY = (previewTop + previewBottom) / 2;
		double unitx = ((double) previewRight - (double) previewleft) / 2000;
		double unity = ((double) previewBottom - (double) previewTop) / 2000;
		int left = clamp((int) (((x - areaWidth / 2) - centerX) / unitx),
				-1000, 1000);
		int top = clamp((int) (((y - areaHeight / 2) - centerY) / unity),
				-1000, 1000);
		int right = clamp((int) (left + areaWidth / unitx), -1000, 1000);
		int bottom = clamp((int) (top + areaHeight / unity), -1000, 1000);

		return new Rect(left, top, right, bottom);
	}

	public int clamp(int x, int min, int max) {
		if (x > max)
			return max;
		if (x < min)
			return min;
		return x;
	}

	protected DisplayMetrics getScreenWH() {
		DisplayMetrics dMetrics = new DisplayMetrics();
		dMetrics = this.getResources().getDisplayMetrics();
		return dMetrics;
	}

	/**
	 * 获取存储相片的路径
	 * @return
	 */
	public static File getOutputMediaFile() {
		// To be safe, you should check that the SDCard is mounted
		// using Environment.getExternalStorageState() before doing this.

		File mediaStorageDir = null;
		try {
			// This location works best if you want the created images to be
			// shared
			// between applications and persist after your app has been
			// uninstalled.
			mediaStorageDir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DCIM),
					PHOTO_FILE_NAME);

		} catch (Exception e) {
			e.printStackTrace();

		}

		// Create the storage directory if it does not exist
		if (!mediaStorageDir.exists()) {
			if (!mediaStorageDir.mkdirs()) {
				// 鍦⊿D鍗′笂鍒涘缓鏂囦欢澶归渶瑕佹潈闄愶細
				// <uses-permission
				// android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
				Log.d("CameraActivity",
						"failed to create directory, check if you have the WRITE_EXTERNAL_STORAGE permission");
				return null;
			}
		}

		// Create a media file name
		String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss")
				.format(new Date());
		File mediaFile;
		mediaFile = new File(mediaStorageDir.getPath() + File.separator
				+ "IMG_" + timeStamp + ".jpg");

		return mediaFile;
	}

	/**
     * 拍照
     */
	public void takePicture() {

		if (camera != null) {

			Log.d("CameraActivity", "camera != null");

			camera.takePicture(null, null, new PictureCallback() {

				@Override
				public void onPictureTaken(byte[] data, Camera camera) {
					// System.out.println("CustomCameraView onPictureTaken " );

					File pictureFile = getOutputMediaFile();

					if (pictureFile == null) {
						Log.d("CameraActivity",
								"Error creating media file, check storage permissions: ");
						return;
					}

					try {
						FileOutputStream fos = new FileOutputStream(pictureFile);
						fos.write(data); //把拍的照片写入对应文件夹
						fos.close();
						
						if (onTakePictureInfo != null) {
							onTakePictureInfo.onTakePictureInofo(true,
									pictureFile);
						}
					} catch (Exception e) {
						if (onTakePictureInfo != null) {
							mode = MODE.NONE;
							onTakePictureInfo.onTakePictureInofo(false, null);
						}
					}

					camera.stopPreview();
					camera.startPreview();
				}
			});

			mode = MODE.NONE;
		}
	}

	public void setOnTakePictureInfo(OnTakePictureInfo _onTakePictureInfo) {
		this.onTakePictureInfo = _onTakePictureInfo;
	}

	public interface OnTakePictureInfo {

		/**
		 * 鎷嶇収鍚庤繑鍥炴媿鐓т俊鎭�
		 *
		 * @param _success 鎷嶇収鎴愬姛
		 * @param _file    鐓х墖鏂囦欢
		 */
		public void onTakePictureInofo(boolean _success, File _file);
	}

	@Override
	public void onAutoFocus(boolean success, Camera _camera) {
		// System.out.println("CustomCameraView onAutoFocus success = " +
		// success);
		if (success) {
			mode = MODE.FOCUSED;
			// Camera.Parameters parameters = _camera.getParameters();
			// float[] output = new float[3];
			// parameters.getFocusDistances(output);
			// List<Camera.Area> area = parameters.getFocusAreas();
			// Rect rect = area.get(0).rect;
			//
			// System.out.println("CustomCameraView onAutoFocus output1 = " +
			// output[1] + " ,output[2] = " + output[2] + " ,output[3]" +
			// output[2]);
			// System.out.println("CustomCameraView onAutoFocus rect.top = " +
			// rect.top + " , rect.left = " + rect.left + " ,rect.right" +
			// rect.right
			// +
			// " ,rect.bottom" + rect.bottom);

			view_focus.setBackground(getResources().getDrawable(
					R.drawable.ic_focus_focused));
		} else {
			mode = MODE.FOCUSFAIL;
			view_focus.setBackground(getResources().getDrawable(
					R.drawable.ic_focus_failed));
		}

		setFocusView();
	}

	private void setFocusView() {
		new Handler().postDelayed(new Runnable() {

			@Override
			public void run() {
				view_focus.setBackground(null);

			}
		}, 1 * 1000);
	}
}
